(* ---------------------------- HARDCOPY.INC ----------------------------------
  Include-File fuer Hardcopies auf dem JOYCE unter Turbo Pascal.
  Ermoeglicht die Ausgabe von Bildschirmbereichen in vier Groesen
  und zwei Richtungen auf dem JOYCE-Drucker.

  Benoetigt das Include-File KERNEL.INC...

------------------------------------------------------------------------------ *)

PROCEDURE Hardcopy (left, top, right, bottom, size : INTEGER; direction : CHAR);

const breite = 719;  (* Bildschirmbreite *)
      hoehe  = 248;  (* Bildschirmhoehe  *)
      ptrres = 960;  (* Breite einer Druckzeile, doppelte Aufloesung *)

      zweih1  : array (.0..7.) OF INTEGER = (128,64,32,16,8,4,2,1);
      zweih2  : array (.0..3.) OF INTEGER = (192,48,12,3);
      zweih4  : array (.0..1.) OF INTEGER = (240,15);
      zweih8  : array (.0..0.) OF INTEGER = (255);

var   bitnumber, i, j, decrement, y, n, spalte  : INTEGER;


FUNCTION params_ok: BOOLEAN;
BEGIN
    IF (left < 0) or (right > breite) or (left > right) or
        (top < 0) or (bottom > hoehe) or (top > bottom) or
        not(size in (.1, 2, 4, 8.)) or
        (bitnumber > ptrres) THEN BEGIN
        params_ok:= false;
        WRITELN('Hardcopy - params_ok: false');
        WRITELN('top: ', top, ' bottom: ', bottom);
        WRITELN('left: ', left, ' right: ', right);
        WRITELN('size: ', size, ' bitnumber: ', bitnumber);
    END
    ELSE
        params_ok:=true;
    REPEAT UNTIL KeyPressed;
END;

BEGIN (* Hardcopy *)
    IF not (direction in (.'v','V'.)) THEN direction := 'h';
    
    CASE direction OF
    'H','h' :
        BEGIN
            bitnumber := size * (right - left +1);
            IF params_ok THEN BEGIN
                y:=top;
                decrement := 8 Div size;
                WRITE (Lst, chr(27), chr(65), chr(8)); (* Grafikzeilenabstand  8/72 *)
                WHILE y <=bottom DO BEGIN
                    WRITE (Lst, chr(27), chr(76), chr(Lo(bitnumber)),
                                                  chr(hi(bitnumber)));
                    (* Anzahl der folgenden Grafikdaten, doppelte Aufloesung *)
                    FOR i:= left TO right DO BEGIN
                        spalte := 0;
                        FOR n := y TO pred(y+decrement) DO
                            IF n <= bottom THEN
                                IF DotColor(i, n) THEN
                                    CASE size OF
                                    1: spalte := spalte or zweih1(.n-y.);
                                    2: spalte := spalte or zweih2(.n-y.);
                                    4: spalte := spalte or zweih4(.n-y.);
                                    8: spalte := spalte or zweih8(.n-y.);
                                    END;
                        FOR j := 1 TO size DO
                            WRITE(Lst, chr(spalte));          (* bits senden *)
                    END;
                    WRITELN(Lst);
                    y := y + decrement;
                END;
                WRITE(Lst, chr(27), chr(50));   (* normaler Zeilenabstand 1/6 *)
            END;
        END;
    'v','V':
        BEGIN
            bitnumber := 2 * size * (bottom - top +1);
            IF params_ok THEN BEGIN
                y:=left;
                decrement := 8 Div  size;
                WRITE (Lst, chr(27), chr(65), chr(8)); (* Grafikzeilenabstand  8/72 *)
                WHILE y <=right DO BEGIN
                    WRITE (Lst, chr(27), chr(76), chr(Lo(bitnumber)),
                                                  chr(hi(bitnumber)));
                    (* Anzahl der folgenden Grafikdaten, doppelte Aufloesung *)
                    FOR i:= bottom DOWNTO top DO BEGIN
                        spalte := 0;
                        FOR n := y TO pred(y+decrement) DO
                            IF n <= right THEN
                                IF DotColor(n,i) THEN
                                    CASE size OF
                                    1: spalte := spalte or zweih1(.n-y.);
                                    2: spalte := spalte or zweih2(.n-y.);
                                    4: spalte := spalte or zweih4(.n-y.);
                                    8: spalte := spalte or zweih8(.n-y.);
                                    END;
                        FOR j := 1 TO 2 * size DO
                            WRITE(Lst, chr(spalte));          (* bits senden *)
                    END;
                    WRITELN(Lst);
                    y := y + decrement;
                END;
                WRITE(Lst, chr(27), chr(50));           (* normaler Zeilenabstand 1/6 *)
            END;
        END;
    END;
END; (* Hardcopy*)

(* ------------------------- Ende von HARDCOPY.INC ---------------------------- *)
